@@ -102,7 +102,8 @@ typedef void (* browser_authentication_cb_t) (struct connman_service *service,
 				bool authentication_done,
 				const char *error, void *user_data);
 int __connman_agent_request_passphrase_input(struct connman_service *service,
-				authentication_cb_t callback, void *user_data);
+				authentication_cb_t callback,
+				const char *dbus_sender, void *user_data);
 int __connman_agent_request_login_input(struct connman_service *service,
 				authentication_cb_t callback, void *user_data);
 int __connman_agent_request_browser(struct connman_service *service,
@@ -222,6 +223,8 @@ int __connman_inet_add_fwmark_rule(uint32_t table_id, int family, uint32_t fwmar
 int __connman_inet_del_fwmark_rule(uint32_t table_id, int family, uint32_t fwmark);
 int __connman_inet_add_default_to_table(uint32_t table_id, int ifindex, const char *gateway);
 int __connman_inet_del_default_from_table(uint32_t table_id, int ifindex, const char *gateway);
+int __connman_inet_get_address_netmask(int ifindex,
+		struct sockaddr_in *address, struct sockaddr_in *netmask);
 
 #include <connman/resolver.h>
 
@@ -272,14 +275,14 @@ struct connman_ipaddress {
 };
 
 struct connman_ipconfig_ops {
-	void (*up) (struct connman_ipconfig *ipconfig);
-	void (*down) (struct connman_ipconfig *ipconfig);
-	void (*lower_up) (struct connman_ipconfig *ipconfig);
-	void (*lower_down) (struct connman_ipconfig *ipconfig);
-	void (*ip_bound) (struct connman_ipconfig *ipconfig);
-	void (*ip_release) (struct connman_ipconfig *ipconfig);
-	void (*route_set) (struct connman_ipconfig *ipconfig);
-	void (*route_unset) (struct connman_ipconfig *ipconfig);
+	void (*up) (struct connman_ipconfig *ipconfig, const char *ifname);
+	void (*down) (struct connman_ipconfig *ipconfig, const char *ifname);
+	void (*lower_up) (struct connman_ipconfig *ipconfig, const char *ifname);
+	void (*lower_down) (struct connman_ipconfig *ipconfig, const char *ifname);
+	void (*ip_bound) (struct connman_ipconfig *ipconfig, const char *ifname);
+	void (*ip_release) (struct connman_ipconfig *ipconfig, const char *ifname);
+	void (*route_set) (struct connman_ipconfig *ipconfig, const char *ifname);
+	void (*route_unset) (struct connman_ipconfig *ipconfig, const char *ifname);
 };
 
 struct connman_ipconfig *__connman_ipconfig_create(int index,
@@ -301,7 +304,6 @@ void *__connman_ipconfig_get_data(struct connman_ipconfig *ipconfig);
 void __connman_ipconfig_set_data(struct connman_ipconfig *ipconfig, void *data);
 
 int __connman_ipconfig_get_index(struct connman_ipconfig *ipconfig);
-const char *__connman_ipconfig_get_ifname(struct connman_ipconfig *ipconfig);
 
 void __connman_ipconfig_set_ops(struct connman_ipconfig *ipconfig,
 				const struct connman_ipconfig_ops *ops);
@@ -513,9 +515,9 @@ int __connman_technology_remove_rfkill(unsigned int index,
 void __connman_technology_scan_started(struct connman_device *device);
 void __connman_technology_scan_stopped(struct connman_device *device);
 void __connman_technology_add_interface(enum connman_service_type type,
-				int index, const char *name, const char *ident);
+				int index, const char *ident);
 void __connman_technology_remove_interface(enum connman_service_type type,
-				int index, const char *name, const char *ident);
+				int index, const char *ident);
 void __connman_technology_notify_regdom_by_device(struct connman_device *device,
 						int result, const char *alpha2);
 
@@ -546,10 +548,6 @@ int __connman_device_disconnect(struct connman_device *device);
 
 bool __connman_device_has_driver(struct connman_device *device);
 
-void __connman_device_set_reconnect(struct connman_device *device,
-						bool reconnect);
-bool __connman_device_get_reconnect(struct connman_device *device);
-
 const char *__connman_device_get_type(struct connman_device *device);
 
 int __connman_rfkill_init(void);
@@ -594,6 +592,8 @@ char **__connman_config_get_string_list(GKeyFile *key_file,
 
 bool __connman_config_get_bool(GKeyFile *key_file,
 	const char *group_name, const char *key, GError **error);
+bool __connman_config_address_provisioned(const char *address,
+					const char *netmask);
 
 int __connman_tethering_init(void);
 void __connman_tethering_cleanup(void);
@@ -615,6 +615,7 @@ int __connman_provider_append_user_route(struct connman_provider *provider,
 			int family, const char *network, const char *netmask);
 void __connman_provider_append_properties(struct connman_provider *provider, DBusMessageIter *iter);
 void __connman_provider_list(DBusMessageIter *iter, void *user_data);
+bool __connman_provider_is_immutable(struct connman_provider *provider);
 int __connman_provider_create_and_connect(DBusMessage *msg);
 const char * __connman_provider_get_ident(struct connman_provider *provider);
 int __connman_provider_indicate_state(struct connman_provider *provider,
@@ -653,7 +654,9 @@ bool __connman_service_is_connected_state(struct connman_service *service,
 					enum connman_ipconfig_type type);
 const char *__connman_service_get_ident(struct connman_service *service);
 const char *__connman_service_get_path(struct connman_service *service);
+const char *__connman_service_get_name(struct connman_service *service);
 unsigned int __connman_service_get_order(struct connman_service *service);
+enum connman_service_state __connman_service_get_state(struct connman_service *service);
 void __connman_service_update_ordering(void);
 struct connman_network *__connman_service_get_network(struct connman_service *service);
 enum connman_service_security __connman_service_get_security(struct connman_service *service);
@@ -694,6 +697,7 @@ int __connman_service_indicate_default(struct connman_service *service);
 int __connman_service_connect(struct connman_service *service);
 int __connman_service_disconnect(struct connman_service *service);
 int __connman_service_disconnect_all(void);
+void __connman_service_set_active_session(bool enable, GSList *list);
 void __connman_service_auto_connect(void);
 bool __connman_service_remove(struct connman_service *service);
 bool __connman_service_is_provider_pending(struct connman_service *service);
@@ -736,9 +740,11 @@ bool __connman_service_is_hidden(struct connman_service *service);
 bool __connman_service_is_split_routing(struct connman_service *service);
 int __connman_service_get_index(struct connman_service *service);
 void __connman_service_set_hidden(struct connman_service *service);
+void __connman_service_set_hostname(struct connman_service *service,
+						const char *hostname);
+const char *__connman_service_get_hostname(struct connman_service *service);
 void __connman_service_set_domainname(struct connman_service *service,
 						const char *domainname);
-const char *__connman_service_get_domainname(struct connman_service *service);
 const char *__connman_service_get_nameserver(struct connman_service *service);
 void __connman_service_set_proxy_autoconfig(struct connman_service *service,
 							const char *url);
@@ -768,14 +774,10 @@ void __connman_service_counter_unregister(const char *counter);
 #include <connman/session.h>
 
 typedef void (* service_iterate_cb) (struct connman_service *service,
-					const char *name,
-					enum connman_service_state state,
 					void *user_data);
 
 int __connman_service_iterate_services(service_iterate_cb cb, void *user_data);
 
-void __connman_service_session_inc(struct connman_service *service);
-bool __connman_service_session_dec(struct connman_service *service);
 void __connman_service_mark_dirty();
 void __connman_service_save(struct connman_service *service);
 
@@ -949,6 +951,7 @@ int __connman_firewall_add_rule(struct firewall_context *ctx,
 				const char *rule_fmt, ...);
 int __connman_firewall_enable(struct firewall_context *ctx);
 int __connman_firewall_disable(struct firewall_context *ctx);
+bool __connman_firewall_is_up(void);
 
 int __connman_firewall_init(void);
 void __connman_firewall_cleanup(void);
@@ -983,6 +986,4 @@ int __connman_nfacct_disable(struct nfacct_context *ctx,
 				connman_nfacct_disable_cb_t cb,
 				void *user_data);
 
-
-int __connman_nfacct_init(void);
 void __connman_nfacct_cleanup(void);
